/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

function _generateTagItem(tag, fn) {
  return [React.createElement(
    'li',
    { key: tag + 0 },
    React.createElement(
      'a',
      { 'data-id': tag, className: 'link', onClick: fn },
      tag
    )
  )];
}

function generateScript(isBundle) {
  let id = 'trusty/django-123';
  if (isBundle) {
    id = 'django-cluster';
  }

  return '<script ' + 'src="https://assets.ubuntu.com/v1/juju-cards-v1.5.0.js"></script>\n' + '<div class="juju-card" data-id="' + id + '"></div>';
}

describe('EntityContent', function () {
  let mockEntity;

  beforeAll(function (done) {
    // By loading these files it makes their classes available in the tests.
    YUI().use('entity-content', function () {
      done();
    });
  });

  beforeEach(function () {
    mockEntity = jsTestUtils.makeEntity();
  });

  afterEach(function () {
    mockEntity = undefined;
  });

  it('can display a charm', function () {
    const apiUrl = 'http://example.com';
    const description = mockEntity.get('description');
    const renderMarkdown = sinon.stub().returns(description);
    const getFile = sinon.spy();
    const changeState = sinon.spy();
    const pluralize = sinon.spy();
    const script = generateScript();
    mockEntity.set('resources', [{ resource: 'one' }]);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: false,
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const option1 = {
      description: 'Your username',
      type: 'string',
      default: 'spinach',
      name: 'username'
    };
    const option2 = {
      description: 'Your password',
      type: 'string',
      default: 'abc123',
      name: 'password'
    };
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'entity-content' },
      React.createElement(
        'div',
        { className: 'row row--grey entity-content__description' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement('div', { className: 'intro',
              dangerouslySetInnerHTML: { __html: description } })
          ),
          React.createElement(
            'div',
            { className: 'four-col entity-content__metadata' },
            React.createElement(
              'h4',
              null,
              'Tags'
            ),
            React.createElement(
              'ul',
              null,
              _generateTagItem('database', instance._handleTagClick)
            )
          )
        )
      ),
      undefined,
      React.createElement(
        'div',
        { className: 'row' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'seven-col append-one' },
            React.createElement(juju.components.EntityContentReadme, {
              entityModel: mockEntity,
              renderMarkdown: renderMarkdown,
              getFile: getFile })
          ),
          React.createElement(
            'div',
            { className: 'four-col' },
            React.createElement(
              'div',
              { className: 'section' },
              React.createElement(
                'h3',
                { className: 'section__title' },
                'Contribute'
              ),
              React.createElement(
                'ul',
                { className: 'section__links' },
                React.createElement(
                  'li',
                  null,
                  React.createElement(
                    'a',
                    { href: 'https://bugs.launchpad.net/charms/+source/django',
                      className: 'link',
                      target: '_blank' },
                    'Submit a bug'
                  )
                ),
                undefined
              )
            ),
            React.createElement(juju.components.EntityResources, {
              apiUrl: apiUrl,
              entityId: mockEntity.get('id'),
              pluralize: pluralize,
              resources: [{ resource: 'one' }] }),
            React.createElement(juju.components.EntityContentRelations, {
              changeState: changeState,
              relations: mockEntity.get('relations') }),
            React.createElement(juju.components.EntityFiles, {
              apiUrl: apiUrl,
              entityModel: mockEntity,
              pluralize: pluralize }),
            React.createElement(juju.components.EntityContentRevisions, {
              revisions: mockEntity.get('revisions') }),
            React.createElement(
              'div',
              { className: 'entity-content__card section clearfix' },
              React.createElement(
                'h3',
                { className: 'section__title' },
                'Embed this charm'
              ),
              React.createElement(
                'p',
                null,
                'Add this card to your website by copying the code below.',
                React.createElement(
                  'a',
                  { className: 'link',
                    href: 'https://jujucharms.com/community/cards',
                    target: '_blank' },
                  'Learn more'
                ),
                '.'
              ),
              React.createElement('textarea', { className: 'twelve-col', cols: '70',
                defaultValue: script, readOnly: 'readonly',
                rows: '2', wrap: 'off' }),
              React.createElement(
                'h4',
                null,
                'Preview'
              ),
              React.createElement('div', { className: 'juju-card', 'data-id': 'trusty/django-123' })
            )
          )
        )
      ),
      React.createElement(
        'div',
        { id: 'configuration',
          className: 'row row--grey entity-content__configuration' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'h2',
              { className: 'entity-content__header' },
              'Configuration'
            ),
            React.createElement(
              'dl',
              null,
              React.createElement(juju.components.EntityContentConfigOption, {
                key: option1.name,
                option: option1 }),
              React.createElement(juju.components.EntityContentConfigOption, {
                key: option2.name,
                option: option2 })
            )
          )
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('can display a charm with terms', function () {
    mockEntity.set('terms', ['term1', 'term2']);
    const showTerms = sinon.stub();
    showTerms.onFirstCall().callsArgWith(2, null, {
      name: 'terms1',
      revision: 5
    });
    showTerms.onSecondCall().callsArgWith(2, null, {
      name: 'terms2',
      revision: 10
    });
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: showTerms,
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const terms = output.props.children[0].props.children.props.children[2];
    const links = terms.props.children[1].props.children;
    const expected = React.createElement(
      'div',
      { className: 'four-col entity-content__metadata' },
      React.createElement(
        'h4',
        null,
        'Terms'
      ),
      React.createElement(
        'ul',
        null,
        [React.createElement(
          'li',
          { className: 'link',
            key: 'terms1',
            onClick: links[0].props.onClick },
          'terms1'
        ), React.createElement(
          'li',
          { className: 'link',
            key: 'terms2',
            onClick: links[1].props.onClick },
          'terms2'
        )]
      )
    );
    expect(terms).toEqualJSX(expected);
  });

  it('can display the terms popup', function () {
    mockEntity.set('terms', ['term1', 'term2']);
    const showTerms = sinon.stub();
    showTerms.onFirstCall().callsArgWith(2, null, {
      name: 'terms1',
      revision: 5
    });
    showTerms.onSecondCall().callsArgWith(2, null, {
      name: 'terms2',
      revision: 10
    });
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: showTerms,
      staticURL: 'http://example.com' }), true);
    const instance = renderer.getMountedInstance();
    let output = renderer.getRenderOutput();
    output.props.children[0].props.children.props.children[2].props.children[1].props.children[1].props.onClick();
    output = renderer.getRenderOutput();
    const expected = React.createElement(juju.components.TermsPopup, {
      close: instance._toggleTerms,
      terms: [{
        name: 'terms2',
        revision: 10
      }] });
    expect(output.props.children[4]).toEqualJSX(expected);
  });

  it('can display a spinner when loading terms', function () {
    mockEntity.set('terms', ['term1', 'term2']);
    const showTerms = sinon.stub();
    showTerms.onFirstCall();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: showTerms,
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'four-col entity-content__metadata' },
      React.createElement(
        'h4',
        null,
        'Terms'
      ),
      React.createElement(juju.components.Spinner, null)
    );
    const terms = output.props.children[0].props.children.props.children[2];
    expect(terms).toEqualJSX(expected);
  });

  it('can handle errors when loading terms', function () {
    mockEntity.set('terms', ['term1', 'term2']);
    const showTerms = sinon.stub().onFirstCall().callsArgWith(2, 'Uh oh', null);
    const addNotification = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: addNotification,
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: showTerms,
      staticURL: 'http://example.com' }), true);
    renderer.getRenderOutput();
    assert.equal(addNotification.callCount, 1);
    assert.deepEqual(addNotification.args[0][0], {
      title: 'Failed to load terms for term1',
      message: 'Failed to load terms for term1: Uh oh',
      level: 'error'
    });
  });

  it('can abort requests when unmounting', function () {
    mockEntity.set('terms', ['term1', 'term2']);
    const abort = sinon.stub();
    const showTerms = sinon.stub().returns({ abort: abort });
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: showTerms,
      staticURL: 'http://example.com' }), true);
    renderer.unmount();
    assert.equal(abort.callCount, 2);
  });

  it('can display a charm with actions', function () {
    mockEntity.set('bugUrl', 'http://example.com/bugs');
    mockEntity.set('homepage', 'http://example.com/');
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'section' },
      React.createElement(
        'h3',
        { className: 'section__title' },
        'Contribute'
      ),
      React.createElement(
        'ul',
        { className: 'section__links' },
        React.createElement(
          'li',
          null,
          React.createElement(
            'a',
            { href: 'http://example.com/bugs',
              className: 'link',
              target: '_blank' },
            'Submit a bug'
          )
        ),
        React.createElement(
          'li',
          null,
          React.createElement(
            'a',
            { href: 'http://example.com/',
              className: 'link',
              target: '_blank' },
            'Project homepage'
          )
        )
      )
    );
    const parent = output.props.children[2].props.children.props.children[1];
    expect(parent.props.children[0]).toEqualJSX(expected);
  });

  it('can display a charm with no options', function () {
    mockEntity.set('options', null);
    const description = mockEntity.get('description');
    const apiUrl = 'http://example.com';
    const renderMarkdown = sinon.stub().returns(description);
    const getFile = sinon.spy();
    const pluralize = sinon.spy();
    const changeState = sinon.spy();
    const script = generateScript();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: false,
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'entity-content' },
      React.createElement(
        'div',
        { className: 'row row--grey entity-content__description' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement('div', { className: 'intro',
              dangerouslySetInnerHTML: { __html: description } })
          ),
          React.createElement(
            'div',
            { className: 'four-col entity-content__metadata' },
            React.createElement(
              'h4',
              null,
              'Tags'
            ),
            React.createElement(
              'ul',
              null,
              _generateTagItem('database', instance._handleTagClick)
            )
          )
        )
      ),
      undefined,
      React.createElement(
        'div',
        { className: 'row' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'seven-col append-one' },
            React.createElement(juju.components.EntityContentReadme, {
              entityModel: mockEntity,
              renderMarkdown: renderMarkdown,
              getFile: getFile })
          ),
          React.createElement(
            'div',
            { className: 'four-col' },
            React.createElement(
              'div',
              { className: 'section' },
              React.createElement(
                'h3',
                { className: 'section__title' },
                'Contribute'
              ),
              React.createElement(
                'ul',
                { className: 'section__links' },
                React.createElement(
                  'li',
                  null,
                  React.createElement(
                    'a',
                    { href: 'https://bugs.launchpad.net/charms/+source/django',
                      className: 'link',
                      target: '_blank' },
                    'Submit a bug'
                  )
                ),
                undefined
              )
            ),
            React.createElement(juju.components.EntityResources, {
              apiUrl: apiUrl,
              entityId: mockEntity.get('id'),
              pluralize: pluralize,
              resources: undefined }),
            React.createElement(juju.components.EntityContentRelations, {
              changeState: changeState,
              relations: mockEntity.get('relations') }),
            React.createElement(juju.components.EntityFiles, {
              apiUrl: apiUrl,
              entityModel: mockEntity,
              pluralize: pluralize }),
            React.createElement(juju.components.EntityContentRevisions, {
              revisions: mockEntity.get('revisions') }),
            React.createElement(
              'div',
              { className: 'entity-content__card section clearfix' },
              React.createElement(
                'h3',
                { className: 'section__title' },
                'Embed this charm'
              ),
              React.createElement(
                'p',
                null,
                'Add this card to your website by copying the code below.',
                React.createElement(
                  'a',
                  { className: 'link',
                    href: 'https://jujucharms.com/community/cards',
                    target: '_blank' },
                  'Learn more'
                ),
                '.'
              ),
              React.createElement('textarea', { className: 'twelve-col', cols: '70',
                defaultValue: script, readOnly: 'readonly',
                rows: '2', wrap: 'off' }),
              React.createElement(
                'h4',
                null,
                'Preview'
              ),
              React.createElement('div', { className: 'juju-card', 'data-id': 'trusty/django-123' })
            )
          )
        )
      ),
      undefined
    );
    expect(output).toEqualJSX(expected);
  });

  it('can display a bundle for Juju 2', function () {
    const apiUrl = 'http://example.com';
    const renderMarkdown = sinon.spy();
    const getFile = sinon.spy();
    const changeState = sinon.spy();
    const pluralize = sinon.spy();
    const mockEntity = jsTestUtils.makeEntity(true, null, false);
    const script = generateScript(true);
    const output = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: false,
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }));
    const expected = React.createElement(
      'div',
      { className: 'entity-content' },
      undefined,
      undefined,
      React.createElement(
        'div',
        { className: 'row' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'seven-col append-one' },
            React.createElement(juju.components.EntityContentReadme, {
              entityModel: mockEntity,
              renderMarkdown: renderMarkdown,
              getFile: getFile })
          ),
          React.createElement(
            'div',
            { className: 'four-col' },
            React.createElement(
              'div',
              { className: 'section' },
              React.createElement(
                'h3',
                { className: 'section__title' },
                'Contribute'
              ),
              React.createElement(
                'ul',
                { className: 'section__links' },
                undefined,
                React.createElement(
                  'li',
                  null,
                  React.createElement(
                    'a',
                    { href: 'https://code.launchpad.net/~charmers/charms/' + 'bundles/django-cluster/bundle',
                      className: 'link',
                      target: '_blank' },
                    'Project homepage'
                  )
                )
              )
            ),
            undefined,
            undefined,
            React.createElement(juju.components.EntityFiles, {
              apiUrl: apiUrl,
              entityModel: mockEntity,
              pluralize: pluralize }),
            React.createElement(juju.components.EntityContentRevisions, {
              revisions: mockEntity.get('revisions') }),
            React.createElement(
              'div',
              { className: 'entity-content__card section clearfix' },
              React.createElement(
                'h3',
                { className: 'section__title' },
                'Embed this charm'
              ),
              React.createElement(
                'p',
                null,
                'Add this card to your website by copying the code below.',
                React.createElement(
                  'a',
                  { className: 'link',
                    href: 'https://jujucharms.com/community/cards',
                    target: '_blank' },
                  'Learn more'
                ),
                '.'
              ),
              React.createElement('textarea', { className: 'twelve-col', cols: '70',
                defaultValue: script, readOnly: 'readonly',
                rows: '2', wrap: 'off' }),
              React.createElement(
                'h4',
                null,
                'Preview'
              ),
              React.createElement('div', { className: 'juju-card', 'data-id': 'django-cluster' })
            )
          )
        )
      ),
      React.createElement(
        'div',
        { id: 'configuration',
          className: 'row row--grey entity-content__configuration' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'h2',
              { className: 'entity-content__header' },
              'Configuration'
            ),
            React.createElement(
              'ul',
              null,
              React.createElement(
                juju.components.ExpandingRow,
                {
                  classes: {
                    'entity-content__bundle-config': true
                  },
                  key: 'gunicorn' },
                React.createElement(
                  'div',
                  { className: 'entity-content__bundle-config-title' },
                  'gunicorn',
                  React.createElement(
                    'div',
                    { className: 'entity-content__bundle-config-chevron' },
                    React.createElement(
                      'div',
                      { className: 'entity-content__bundle-config-expand' },
                      React.createElement(juju.components.SvgIcon, {
                        name: 'chevron_down_16',
                        size: '16' })
                    ),
                    React.createElement(
                      'div',
                      { className: 'entity-content__bundle-config-contract' },
                      React.createElement(juju.components.SvgIcon, {
                        name: 'chevron_up_16',
                        size: '16' })
                    )
                  )
                ),
                React.createElement(
                  'dl',
                  { className: 'entity-content__bundle-config-options' },
                  [React.createElement(
                    'div',
                    { className: 'entity-content__config-option',
                      key: 'name0' },
                    React.createElement(
                      'dt',
                      { className: 'entity-content__config-name' },
                      'name'
                    ),
                    React.createElement(
                      'dd',
                      { className: 'entity-content__config-description' },
                      React.createElement(
                        'p',
                        null,
                        'title'
                      )
                    )
                  ), React.createElement(
                    'div',
                    { className: 'entity-content__config-option',
                      key: 'active1' },
                    React.createElement(
                      'dt',
                      { className: 'entity-content__config-name' },
                      'active'
                    ),
                    React.createElement(
                      'dd',
                      { className: 'entity-content__config-description' },
                      React.createElement(
                        'p',
                        null,
                        true
                      )
                    )
                  )]
                )
              ),
              React.createElement(
                juju.components.ExpandingRow,
                {
                  classes: {
                    'entity-content__bundle-config': true
                  },
                  key: 'django' },
                React.createElement(
                  'div',
                  { className: 'entity-content__bundle-config-title' },
                  'django',
                  React.createElement(
                    'div',
                    { className: 'entity-content__bundle-config-chevron' },
                    React.createElement(
                      'div',
                      { className: 'entity-content__bundle-config-expand' },
                      React.createElement(juju.components.SvgIcon, {
                        name: 'chevron_down_16',
                        size: '16' })
                    ),
                    React.createElement(
                      'div',
                      { className: 'entity-content__bundle-config-contract' },
                      React.createElement(juju.components.SvgIcon, {
                        name: 'chevron_up_16',
                        size: '16' })
                    )
                  )
                ),
                React.createElement(
                  'dl',
                  { className: 'entity-content__bundle-config-options' },
                  [React.createElement(
                    'div',
                    { key: 'none' },
                    'Config options not modified in this bundle.'
                  )]
                )
              )
            )
          )
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('can display a bundle with actions', function () {
    mockEntity = jsTestUtils.makeEntity(true, null, false);
    mockEntity.set('bugUrl', 'http://example.com/bugs');
    mockEntity.set('homepage', 'http://example.com/');
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: sinon.stub(),
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'section' },
      React.createElement(
        'h3',
        { className: 'section__title' },
        'Contribute'
      ),
      React.createElement(
        'ul',
        { className: 'section__links' },
        React.createElement(
          'li',
          null,
          React.createElement(
            'a',
            { href: 'http://example.com/bugs',
              className: 'link',
              target: '_blank' },
            'Submit a bug'
          )
        ),
        React.createElement(
          'li',
          null,
          React.createElement(
            'a',
            { href: 'http://example.com/',
              className: 'link',
              target: '_blank' },
            'Project homepage'
          )
        )
      )
    );
    const parent = output.props.children[2].props.children.props.children[1];
    expect(parent.props.children[0]).toEqualJSX(expected);
  });

  it('doesn\'t show relations when they don\'t exist', function () {
    const apiUrl = 'http://example.com';
    const renderMarkdown = sinon.spy();
    const getFile = sinon.spy();
    const changeState = sinon.spy();
    const pluralize = sinon.spy();
    mockEntity.set('relations', { requires: {}, provides: {} });
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: false,
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const parent = output.props.children[2].props.children.props.children[1];
    const relationsComponent = parent.props.children[2];
    assert.equal(relationsComponent, undefined);
  });

  it('can display plans', function () {
    const plans = [{
      url: 'plan1',
      price: 'test/price1',
      description: 'description1'
    }, {
      url: 'plan2',
      price: 'price2;',
      description: 'description2'
    }, {
      url: 'plan3',
      price: 'test/price3;price3b',
      description: 'description3'
    }];
    mockEntity.set('options', null);
    const apiUrl = 'http://example.com';
    const renderMarkdown = sinon.spy();
    const getFile = sinon.spy();
    const pluralize = sinon.spy();
    const changeState = sinon.spy();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: true,
      plans: plans,
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { id: 'plans',
        className: 'row entity-content__plans' },
      React.createElement(
        'div',
        { className: 'inner-wrapper' },
        React.createElement(
          'div',
          { className: 'twelve-col' },
          React.createElement(
            'h2',
            { className: 'entity-content__header' },
            'Plans'
          ),
          React.createElement(
            'div',
            { className: 'equal-height' },
            [React.createElement(
              'div',
              { className: 'entity-content__plan four-col',
                key: 'plan10' },
              React.createElement(
                'div',
                { className: 'entity-content__plan-content' },
                React.createElement(
                  'h3',
                  { className: 'entity-content__plan-title' },
                  'plan1'
                ),
                React.createElement(
                  'ul',
                  { className: 'entity-content__plan-price' },
                  [React.createElement(
                    'li',
                    { className: 'entity-content__plan-price-item',
                      key: 'testprice10' },
                    React.createElement(
                      'span',
                      { className: 'entity-content__plan-price-amount' },
                      'test'
                    ),
                    React.createElement(
                      'span',
                      { className: 'entity-content__plan-price-quantity' },
                      '/ ',
                      'price1'
                    )
                  )]
                ),
                React.createElement(
                  'p',
                  { className: 'entity-content__plan-description' },
                  'description1'
                )
              )
            ), React.createElement(
              'div',
              { className: 'entity-content__plan four-col',
                key: 'plan21' },
              React.createElement(
                'div',
                { className: 'entity-content__plan-content' },
                React.createElement(
                  'h3',
                  { className: 'entity-content__plan-title' },
                  'plan2'
                ),
                React.createElement(
                  'ul',
                  { className: 'entity-content__plan-price' },
                  [React.createElement(
                    'li',
                    { className: 'entity-content__plan-price-item',
                      key: 'price20' },
                    React.createElement(
                      'span',
                      { className: 'entity-content__plan-price-amount' },
                      'price2'
                    ),
                    undefined
                  )]
                ),
                React.createElement(
                  'p',
                  { className: 'entity-content__plan-description' },
                  'description2'
                )
              )
            ), React.createElement(
              'div',
              { className: 'entity-content__plan four-col last-col',
                key: 'plan32' },
              React.createElement(
                'div',
                { className: 'entity-content__plan-content' },
                React.createElement(
                  'h3',
                  { className: 'entity-content__plan-title' },
                  'plan3'
                ),
                React.createElement(
                  'ul',
                  { className: 'entity-content__plan-price' },
                  React.createElement(
                    'li',
                    { className: 'entity-content__plan-price-item',
                      key: 'testprice30' },
                    React.createElement(
                      'span',
                      { className: 'entity-content__plan-price-amount' },
                      'test'
                    ),
                    React.createElement(
                      'span',
                      { className: 'entity-content__plan-price-quantity' },
                      '/ ',
                      'price3'
                    )
                  ),
                  React.createElement(
                    'li',
                    { className: 'entity-content__plan-price-item',
                      key: 'price3b1' },
                    React.createElement(
                      'span',
                      { className: 'entity-content__plan-price-amount' },
                      'price3b'
                    ),
                    undefined
                  )
                ),
                React.createElement(
                  'p',
                  { className: 'entity-content__plan-description' },
                  'description3'
                )
              )
            )]
          )
        )
      )
    );
    expect(output.props.children[1]).toEqualJSX(expected);
  });

  it('can display loading plans', function () {
    mockEntity.set('options', null);
    const apiUrl = 'http://example.com';
    const renderMarkdown = sinon.spy();
    const getFile = sinon.spy();
    const pluralize = sinon.spy();
    const changeState = sinon.spy();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: true,
      plans: null,
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(juju.components.Spinner, null);
    expect(output.props.children[1]).toEqualJSX(expected);
  });

  it('can remove plans when none exist', function () {
    mockEntity.set('options', null);
    const apiUrl = 'http://example.com';
    const renderMarkdown = sinon.spy();
    const getFile = sinon.spy();
    const pluralize = sinon.spy();
    const changeState = sinon.spy();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: apiUrl,
      changeState: changeState,
      entityModel: mockEntity,
      getFile: getFile,
      hasPlans: true,
      plans: [],
      pluralize: pluralize,
      renderMarkdown: renderMarkdown,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }), true);
    const output = renderer.getRenderOutput();
    const expected = undefined;
    assert.deepEqual(output.props.children[1], expected);
  });

  it('can render markdown in the description', function () {
    // Note that this functional test is just a sanity check, not a
    // comprehensive test of the markdown syntax.
    mockEntity.set('description', 'A simple [link](http://google.com/).');
    const output = jsTestUtils.shallowRender(React.createElement(juju.components.EntityContent, {
      addNotification: sinon.stub(),
      apiUrl: 'http://example.com',
      changeState: sinon.stub(),
      entityModel: mockEntity,
      getFile: sinon.stub(),
      hasPlans: false,
      pluralize: sinon.stub(),
      renderMarkdown: marked,
      showTerms: sinon.stub(),
      staticURL: 'http://example.com' }));
    const description = output.props.children[0].props.children.props.children[0].props.children;
    const markupObject = description.props.dangerouslySetInnerHTML;
    assert.equal(markupObject.__html, '<p>A simple <a href="http://google.com/">link</a>.</p>\n');
  });
});