/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('UserMenu', () => {

  beforeAll(done => {
    // By loading this file it adds the component to the juju components.
    YUI().use('user-menu', function () {
      done();
    });
  });

  const LogoutLink = React.createElement('div', null);
  function createEle(props, wrappedComponent) {
    props = props || {};
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.UserMenu.prototype.wrappedComponent, {
      LogoutLink: props.LogoutLink || {},
      USSOLoginLink: props.USSOLoginLink || {},
      controllerAPI: props.controllerAPI || {},
      navigateUserAccount: props.navigateUserAccount || sinon.stub(),
      navigateUserProfile: props.navigateUserProfile || sinon.stub()
    }), true);
    return {
      renderer: renderer,
      output: renderer.getRenderOutput(),
      instance: renderer.getMountedInstance()
    };
  }

  it('renders login text when user not authed', () => {
    const userMenu = createEle();
    const expected = React.createElement(
      'div',
      { className: 'header-menu' },
      React.createElement(
        'span',
        { className: 'header-menu__button header-menu__button-with-text',
          onClick: userMenu.instance.toggleUserMenu,
          role: 'button',
          tabIndex: '0',
          'aria-haspopup': 'true',
          'aria-owns': 'userMenu',
          'aria-controls': 'userMenu',
          'aria-expanded': 'false' },
        {}
      ),
      undefined
    );
    assert.deepEqual(userMenu.output, expected);
  });

  it('renders a user icon when user is authed', () => {
    const userMenu = createEle({
      LogoutLink: LogoutLink,
      controllerAPI: {
        userIsAuthenticated: sinon.stub().returns(true)
      }
    });
    const expected = React.createElement(
      'div',
      { className: 'header-menu' },
      React.createElement(
        'span',
        { className: 'header-menu__button',
          onClick: userMenu.instance.toggleUserMenu,
          role: 'button',
          tabIndex: '0',
          'aria-haspopup': 'true',
          'aria-owns': 'userMenu',
          'aria-controls': 'userMenu',
          'aria-expanded': 'false' },
        React.createElement(juju.components.SvgIcon, { name: 'user_16',
          className: 'header-menu__icon',
          size: '16' })
      ),
      userMenu.instance._generateUserMenu()
    );
    assert.deepEqual(userMenu.output, expected);
  });

  describe('menu', () => {
    it('opens a menu when clicked', () => {
      // TODO frankban: remove juju_config handling when deleting the flag.
      const jujuConfig = window.juju_config;
      window.juju_config = { accountFlag: true };

      const userMenu = createEle({
        LogoutLink: LogoutLink,
        controllerAPI: {
          userIsAuthenticated: sinon.stub().returns(true)
        }
      });
      userMenu.instance.toggleUserMenu();
      const output = userMenu.renderer.getRenderOutput();

      assert.equal(output.props.children.length, 2);
      assert.deepEqual(output.props.children[0].props.className, 'header-menu__button header-menu__show-menu');

      // TODO frankban: remove juju_config handling when deleting the flag.
      window.juju_config = jujuConfig;

      const expectedOutput = React.createElement(
        juju.components.Panel,
        {
          instanceName: 'header-menu__menu',
          visible: true },
        React.createElement(
          'ul',
          { className: 'header-menu__menu-list', role: 'menubar' },
          React.createElement(
            'li',
            { className: 'header-menu__menu-list-item header-menu__menu-list-item-with-link',
              role: 'menuitem', tabIndex: '0' },
            React.createElement(
              'a',
              { className: 'header-menu__menu-list-item-link',
                role: 'button',
                onClick: userMenu.instance._handleProfileClick },
              'Profile'
            )
          ),
          React.createElement(
            'li',
            { className: 'header-menu__menu-list-item header-menu__menu-list-item-with-link',
              role: 'menuitem', tabIndex: '0' },
            React.createElement(
              'a',
              { className: 'header-menu__menu-list-item-link',
                role: 'button',
                onClick: userMenu.instance._handleAccountClick },
              'Account'
            )
          ),
          React.createElement(
            'li',
            { className: 'header-menu__menu-list-item header-menu__menu-list-item-with-link',
              role: 'menuitem', tabIndex: '0' },
            LogoutLink
          )
        )
      );
      expect(output.props.children[1]).toEqualJSX(expectedOutput);
    });

    it('closes when handleClickOutside is called', () => {
      const userMenu = createEle({
        LogoutLink: LogoutLink,
        controllerAPI: {
          userIsAuthenticated: sinon.stub().returns(true)
        }
      });
      userMenu.instance.toggleUserMenu();
      let output = userMenu.renderer.getRenderOutput();

      assert.equal(output.props.children.length, 2);
      assert.deepEqual(output.props.children[0].props.className, 'header-menu__button header-menu__show-menu');
      assert.isDefined(output.props.children[1]);

      userMenu.instance.handleClickOutside();
      output = userMenu.renderer.getRenderOutput();
      assert.equal(output.props.children.length, 2);
      assert.deepEqual(output.props.children[0].props.className, 'header-menu__button');
      assert.deepEqual(output.props.children[1], '');
    });
  });
});